/*

scanner.lex
  -- this is a JLEX specification file for IAparser tokens,
     as part of the "ia_parser library" version 0.1beta1, 10/97

Copyright (C) 1997 Timothy J. Hickey

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

 */



package ia_parser;

import java.lang.System;

class Utility {
  public static void assert
    (
     boolean expr
     )
      { 
	if (false == expr) {
	  throw (new Error("Error: Assertion failed."));
	}
      }
   
 private static final String errorMsg[] = {
    "Error: Unmatched end-of-comment punctuation.",
    "Error: Unmatched start-of-comment punctuation.",
    "Error: Unclosed string.",
    "Error: Illegal character."
    };
  
  public static final int E_ENDCOMMENT = 0; 
  public static final int E_STARTCOMMENT = 1; 
  public static final int E_UNCLOSEDSTR = 2; 
  public static final int E_UNMATCHED = 3; 

  public static void error
    (
     int code
     )
      {
	System.out.println(errorMsg[code]);
      }
}


class Yytoken {
  Yytoken 
    (
     int index,
     String text,
     int line,
     int charBegin,
     int charEnd
     )
      {
	m_index = index;
	m_text = new String(text);
	m_line = line;
	m_charBegin = charBegin;
	m_charEnd = charEnd;
      }

  public int m_index;
  public String m_text;
  public int m_line;
  public int m_charBegin;
  public int m_charEnd;
}

%%

%{
  private int comment_count = 0;
%} 
%line
%char
%state COMMENT

ALPHA=[A-Za-z]
DIGIT=[0-9"."]
NONNEWLINE_WHITE_SPACE_CHAR=[\ \t\b\012]
WHITE_SPACE_CHAR=[\n\ \t\b\012]
STRING_TEXT=(\\\"|[^\n\"]|\\{WHITE_SPACE_CHAR}+\\)*
COMMENT_TEXT=([^/*\n]|[^*\n]"/"[^*\n]|[^/\n]"*"[^/\n]|"*"[^/\n]|"/"[^*\n])*

%eofval{
        {
         return (new Yytoken(sym.EOF, yytext(),yyline,yychar,yychar+1));
        }
%eofval}


%% 
<YYINITIAL> "MP" { return (new Yytoken(sym.MP,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "LP" { return (new Yytoken(sym.LP,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "RP" { return (new Yytoken(sym.RP,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "narrow" { return (new Yytoken(sym.NARROW,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "-infty" { return (new Yytoken(sym.NEGINFTY,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "infty" { return (new Yytoken(sym.INFTY,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "exp" { return (new Yytoken(sym.EXP,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "log" { return (new Yytoken(sym.LOG,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "sin" { return (new Yytoken(sym.SIN,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "cos" { return (new Yytoken(sym.COS,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "tan" { return (new Yytoken(sym.TAN,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "asin" { return (new Yytoken(sym.ASIN,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "acos" { return (new Yytoken(sym.ACOS,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "atan" { return (new Yytoken(sym.ATAN,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "^" { return (new Yytoken(sym.CAROT,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "**" { return (new Yytoken(sym.POWER,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> ";" { return (new Yytoken(sym.SEMI,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "," { return (new Yytoken(sym.COMMA,  yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "(" { return (new Yytoken(sym.LPAREN,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> ")" { return (new Yytoken(sym.RPAREN,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "[" { return (new Yytoken(sym.LBRACKET,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "]" { return (new Yytoken(sym.RBRACKET,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "+" { return (new Yytoken(sym.PLUS,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "-" { return (new Yytoken(sym.MINUS,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "*" { return (new Yytoken(sym.TIMES,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "/" { return (new Yytoken(sym.DIVIDE,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "=" { return (new Yytoken(sym.EQUALS,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "!=" { return (new Yytoken(sym.NE,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> ">"  { return (new Yytoken(sym.GT,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> ">=" { return (new Yytoken(sym.GE,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "<"  { return (new Yytoken(sym.LT,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "<=" { return (new Yytoken(sym.LE,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> ":=" { return (new Yytoken(sym.COLON_EQUALS,yytext(),yyline,yychar,yychar+2)); }

<YYINITIAL> {NONNEWLINE_WHITE_SPACE_CHAR}+ { }

<YYINITIAL,COMMENT> \n { }

<YYINITIAL> "/*" { yybegin(COMMENT); comment_count = comment_count + 1; }

<COMMENT> "/*" { comment_count = comment_count + 1; }

<COMMENT> "*/" { 
	comment_count = comment_count - 1; 
	Utility.assert(comment_count >= 0);
	if (comment_count == 0) {
    		yybegin(YYINITIAL);
	}
}
<COMMENT> {COMMENT_TEXT} { }

<YYINITIAL> \"{STRING_TEXT}\" {
	String str =  yytext().substring(1,yytext().length() - 1);
	
	Utility.assert(str.length() == yytext().length() - 2);
	return (new Yytoken(sym.error,str,yyline,yychar,yychar + str.length()));
}
<YYINITIAL> \"{STRING_TEXT} {
	String str =  yytext().substring(1,yytext().length());

	Utility.error(Utility.E_UNCLOSEDSTR);
	Utility.assert(str.length() == yytext().length() - 1);
	return (new Yytoken(sym.error,str,yyline,yychar,yychar + str.length()));
} 
<YYINITIAL> {DIGIT}+ {
	return (new Yytoken(sym.NUMBER,
                yytext(),yyline,yychar,yychar + yytext().length()));
}
<YYINITIAL> {ALPHA}({ALPHA}|{DIGIT}|_)* {
	return (new Yytoken(sym.VARIABLE,yytext(),yyline,yychar,yychar + yytext().length()));
}	
<YYINITIAL,COMMENT> . {
        System.out.println("Illegal character: <" + yytext() + ">");
	Utility.error(Utility.E_UNMATCHED);
}







